from vietocr.tool.predictor import Predictor
from vietocr.tool.config import Cfg
from PIL import Image
from robot.api.deco import keyword, not_keyword
import cv2
import json
from openpyxl import Workbook
from datetime import datetime

from src.EduRPA.transform import perspective_transform

class DocumentKeywords:
    def __init__(self, lang, performance, *args, **kwargs):
        cfg = Cfg.load_config_from_name("vgg_seq2seq")
        cfg["device"] = "cpu"
        vgg_seq2seq = Predictor(cfg)

        cfg = Cfg.load_config_from_name("vgg_transformer")
        cfg["device"] = "cpu"
        vgg_transformer = Predictor(cfg)
        
        self.config = {
            "vi": {
              "fast": {
                "predictor": vgg_seq2seq
              },
              "accurate": {
                "predictor": vgg_transformer
              }
            },
            "en": {
                
            }
        }
        self.predictor = self.config[lang][performance]["predictor"]

    @not_keyword
    def image_preprocessing(self, image):
        processed_image = perspective_transform(image)
        return processed_image   
    
    @not_keyword
    def extract(self, labeled_images):
        results = {}
        for label, image in labeled_images.items():
            print("Predict Image", label)
            prediction = self.predictor.predict(Image.fromarray(image))
            results[label] = prediction

        return results
    
    @keyword("Extract Data From Document")
    def extract_data_from_document(self, file_name, template):
        if type (template) is str:
            template = json.loads(template)
        
        # read and preprocess document (image type)
        document = cv2.imread(file_name)
        document = self.image_preprocessing(document)

        # extract images from document
        extracted_labeled_images = {}
        for key in template.keys():
            x1, y1, x2, y2 = list(template[key])
            extracted_labeled_images[key] = []
            extracted_labeled_images[key] = document[y1:y2, x1:x2]

        # extract data from images
        return self.extract(extracted_labeled_images)
    
    @keyword("Create Grade Report File", types={'correct_answer': dict, 'actual_answers': list[dict], 'file_names': list})
    def create_grade_report_file(self, correct_answer, actual_answers, file_names):
        # Initialize report file
        workbook = Workbook()
        report_file_name = f'report_{datetime.now().strftime("%Y%m%d_%H%M%S")}.xlsx'
        summarizeSheet = workbook.create_sheet(title=f"Summary")
        header = ['Student Name'] + [label for label in correct_answer.keys()] + ['Total Score']
        summarizeSheet.append(header)

        for (file_name, actual_answer) in zip(file_names, actual_answers):
            summary = []
            lname = file_name.split('.')
            lname.pop()
            lname = '_'.join(lname)
            summary.append(lname)

            sheet = workbook.create_sheet(title=f"Result_{lname}")

            # Write header
            header = ['Label', 'Actual answer', 'Correct answer', 'Score']
            sheet.append(header)

            # Write data rows
            total_score = 0
            for (label , correct_answer_label, actual_answer_label) in zip(correct_answer.keys(), correct_answer.values(), actual_answer.values()):
                is_correct = correct_answer_label == actual_answer_label
                row = [label, actual_answer_label, correct_answer_label, is_correct]
                sheet.append(row)
                
                summary.append(actual_answer_label)
                total_score += is_correct

            sheet.append(['','','Total',total_score])
            summary.append(total_score)

            summarizeSheet.append(summary)

        # Remove the default sheet created by openpyxl
        workbook.remove(workbook['Sheet'])

        # Save the Excel file
        workbook.save(report_file_name)

        return report_file_name
    


import cv2
import json
from robot.api import logger
from robot.api.deco import keyword, not_keyword

class OpenCVAnnotationLibrary:
    def __init__(self):
        # Global variables
        self.annotations = []
        self.current_annotation = None
        self.image = None  # Initialize the image variable
        self.selected_annotation = None

        # Colors
        self.annotation_color = (0, 255, 0)  # Initial color (green) for annotations
        self.selected_color = (0, 0, 255)  # Color (red) for the selected annotation

    # Mouse callback function
    @not_keyword
    def draw_bbox(self, event, x, y, flags, param):
        if event == cv2.EVENT_LBUTTONDOWN:
            self.current_annotation = {"x1": x, "y1": y}
            self.selected_annotation = None  # Clear selected annotation

        elif event == cv2.EVENT_LBUTTONUP:
            self.current_annotation["x2"] = x
            self.current_annotation["y2"] = y
            self.annotations.append({"bbox": self.current_annotation.copy(), "color": self.annotation_color})
            self.current_annotation.clear()

        elif event == cv2.EVENT_MOUSEMOVE and self.current_annotation:
            temp_image = self.image.copy()
            cv2.rectangle(temp_image, (self.current_annotation["x1"], self.current_annotation["y1"]),
                          (x, y), self.annotation_color, 2)
            cv2.imshow("Image", temp_image)

        # Check if the click is within an existing annotation
        if event == cv2.EVENT_RBUTTONDOWN:
            for idx, annotation in enumerate(self.annotations):
                bbox = annotation["bbox"]
                x1, y1, x2, y2 = bbox["x1"], bbox["y1"], bbox["x2"], bbox["y2"]
                if x1 <= x <= x2 and y1 <= y <= y2:
                    self.selected_annotation = idx
                    break
    @not_keyword
    def render_annotations(self):
        for idx, annotation in enumerate(self.annotations):
            bbox = annotation["bbox"]
            color = self.selected_color if self.selected_annotation == idx else annotation["color"]
            x1, y1, x2, y2 = bbox["x1"], bbox["y1"], bbox["x2"], bbox["y2"]
            cv2.rectangle(self.image, (x1, y1), (x2, y2), color, 2)

    @keyword("Open Annotate Bounding Box")
    def main(self, image_path):
        print(image_path)
        image = cv2.imread(image_path)
        self.image = image.copy()
        cv2.imshow("Image", image)
        cv2.setMouseCallback("Image", self.draw_bbox)

        while True:
            key = cv2.waitKey(1) & 0xFF
            if key == ord('s'):
                annotation_file = "annotations.json"
                with open(annotation_file, 'w') as json_file:
                    json.dump(self.annotations, json_file, indent=4)
                logger.info(f"Annotations saved to {annotation_file}")
                break
            elif key == ord('d'):
                if self.selected_annotation is not None:
                    logger.info(self.selected_annotation)
                    removed_annotation = self.annotations.pop(self.selected_annotation)
                    del removed_annotation
                    self.selected_annotation = None
            elif key == ord('q'):
                break

            # Render the annotations on the image and continuously update the display
            self.render_annotations()
            cv2.imshow("Image", self.image)

        cv2.destroyAllWindows()
        return image, [box["bbox"] for box in self.annotations]
    @keyword("Save Annotation")
    def save_annotations(self, filename):
        with open(filename, 'w') as json_file:
            json.dump(self.annotations, json_file, indent=4)
        logger.info(f"Annotations saved to {filename}")

    @not_keyword
    def delete_selected_annotation(self):
        if self.selected_annotation is not None:
            logger.info(self.selected_annotation)
            self.annotations.pop(self.selected_annotation)
            self.selected_annotation = None

    def quit_application(self):
        cv2.destroyAllWindows()




from robot.api.deco import keyword, not_keyword
from openpyxl import Workbook
from datetime import datetime

@keyword("Make Grade Report", types={'answers': list, 'filename': list ,'results': list[list]})
def make_report(answers, filename ,results):
    workbook = Workbook()
    report_file = f'report_{datetime.now().strftime("%Y%m%d_%H%M%S")}.xlsx'
    summarizeSheet = workbook.create_sheet(title=f"Summary")
    header = ['Name'] + [str(i) for i in range(1, len(answers)+1)]+ ['Score']
    summarizeSheet.append(header)

    for (name, result) in zip(filename, results):
        summary = []
        lname = name.split('.')
        lname.pop()
        lname = '_'.join(lname)
        summary.append(lname)

        sheet = workbook.create_sheet(title=f"Result_{lname}")

        # Write header
        header = ['No', 'Answer', 'Result', 'Correct']
        sheet.append(header)

        # Write data rows
        scores = 0
        for(i,ans, res) in zip(range(1, len(answers)+1),answers, result):
            validation = ans == res
            row = [i,ans,res, validation]
            sheet.append(row)
            summary.append(res)
            scores += validation
        sheet.append(['','','Total',scores])
        summary.append(scores)

        summarizeSheet.append(summary)

    # Remove the default sheet created by openpyxl
    workbook.remove(workbook['Sheet'])

    # Save the Excel file
    workbook.save(report_file)

    return report_file




from vietocr.tool.predictor import Predictor
from vietocr.tool.config import Cfg
import numpy as np
from PIL import Image
from robot.api.deco import keyword, not_keyword
import cv2
import json
class HandwrittenOCR:
    def __init__(self, lang, performance, *args, **kwargs):
        cfg = Cfg.load_config_from_name("vgg_seq2seq")
        cfg["device"] = "cpu"
        # cfg['weights'] = './weights/vgg_seq2seq.pth'
        vgg_seq2seq = Predictor(cfg)

        cfg = Cfg.load_config_from_name("vgg_transformer")
        # cfg['weights'] = './weights/vgg_transfer.pth'
        cfg["device"] = "cpu"
        vgg_transformer = Predictor(cfg)
        
        self.config = {
            "vi": {
              "fast": {
                "predictor": vgg_seq2seq
              },
              "accurate": {
                "predictor": vgg_transformer
              }
            },
            "en": {
                
            }
        }
        self.predictor = self.config[lang][performance]["predictor"]

    @not_keyword
    def image_preprocessing(self, image):
        gray_image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        _, thresh_image = cv2.threshold(gray_image, 0, 255, cv2.THRESH_BINARY_INV + cv2.THRESH_OTSU)
        return thresh_image   
    
    @not_keyword
    def extract(self, images):
        predictor = self.predictor
        def predict(image):
            result = predictor.predict(Image.fromarray(image))
            return result
        results = []
        for image in images:
            print("Predict Image", image)
            prediction = predict(image)
            results.append(prediction)
        return results
    
    @keyword("Extract Text From Image Path")
    def detect(self, image_path, bounding_boxes):
        if(type (bounding_boxes) is str):
            bounding_boxes = json.loads(bounding_boxes)
        image = cv2.imread(image_path)
        extracted_images = []
        for bbox in bounding_boxes:
            x1, y1, x2, y2 = list(bbox)
            extracted_images.append(image[y1:y2, x1:x2])
        return self.extract(extracted_images)

    


import cv2
import numpy as np

def perspective_transform(image):
    # Convert the image to grayscale
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    # Apply GaussianBlur to reduce noise and improve contour detection
    blurred = cv2.GaussianBlur(gray, (5, 5), 0)

    # Use Canny edge detector to find edges
    edges = cv2.Canny(blurred, 50, 150)

    # Find contours in the edged image
    contours, _ = cv2.findContours(edges, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

    # Find the contour with four corners (assuming the document is the largest contour)
    largest_contour = max(contours, key=cv2.contourArea)

    # Approximate the contour to a quadrilateral
    epsilon = 0.02 * cv2.arcLength(largest_contour, True)
    approx = cv2.approxPolyDP(largest_contour, epsilon, True)

    # Check if the contour is a quadrilateral
    if len(approx) != 4:
        raise Exception('The object in the image is not a document or not a clear picture.')

    # Rearrange the order of the points in the contour to be clockwise from the top left
    src = np.zeros((4, 1, 2), dtype="float32")
    for i in range(4):
        src[i] = approx[i]   
    dst, size = infer_dst(src)
    src = rearrange(src)

    # Perform perspective transformation to flatten the document
    matrix = cv2.getPerspectiveTransform(src, dst)
    result = cv2.warpPerspective(image, matrix, size)

    return result

def infer_dst(src_pts):
    dst_pts = np.zeros((4, 2), dtype="float32")
    size = np.zeros((2), dtype="float32")
    
    max_x, min_x, max_y, min_y = get_max_min_x_y(src_pts)

    if max_x - min_x > max_y - min_y:
        dst_pts[0] = [0, 0]
        dst_pts[1] = [0, 600]
        dst_pts[2] = [800, 600]
        dst_pts[3] = [800, 0]
        size = (800, 600)
    else:
        dst_pts[0] = [0, 0]
        dst_pts[1] = [0, 800]
        dst_pts[2] = [600, 800]
        dst_pts[3] = [600, 0]
        size = (600, 800)
    
    return dst_pts, size

def rearrange(src_pts):
    result = np.zeros((4, 1, 2), dtype="float32")

    max_x, min_x, max_y, min_y = get_max_min_x_y(src_pts)
    rec_size = (max_x - min_x, max_y - min_y)

    for i in range(4):
        if min_x <= src_pts[i][0][0] <= min_x + rec_size[0] / 2:
            if min_y <= src_pts[i][0][1] <= min_y + rec_size[1] / 2:
                result[0][0] = src_pts[i][0]
            else:
                result[1][0] = src_pts[i][0]
        else:
            if min_y <= src_pts[i][0][1] <= min_y + rec_size[1] / 2:
                result[3][0] = src_pts[i][0]
            else:
                result[2][0] = src_pts[i][0]

    return result

def get_max_min_x_y(pts):
    max_x = max(pts[:, :, 0])
    min_x = min(pts[:, :, 0])
    max_y = max(pts[:, :, 1])
    min_y = min(pts[:, :, 1])
    return max_x, min_x, max_y, min_y
